using System;
using System.Collections;
using System.Reflection;

[assembly:AssemblyVersion("2.0.0.0")]
[assembly:AssemblyKeyFile("MyCompany.keys")]

namespace MyBankSoftware {
	
	public class BankOperationException : ApplicationException {
		public string AccountNum;
		public string Operation;
		
		public BankOperationException(string AccountNum, string Operation, string ErrMessage) : base(ErrMessage) {
			this.AccountNum = AccountNum;
			this.Operation = Operation;
		}
	}
	
	internal class Account {
		protected readonly string strAccountNum;
		protected double dblAccountBalance;
		
		public Account(string strAccountNum) {
			if (strAccountNum == null) {
				throw new BankOperationException(strAccountNum, "Account.Constructor", "strAccountNum is null!");
			} else {
				this.strAccountNum = strAccountNum;
				dblAccountBalance = 0;
			}
		}
		
		public string AccountNum {
			get {
				return strAccountNum;
			}
		}
		
		public double AccountBalance {
			get {
				return dblAccountBalance;
			}
		}
		
		public void takeMoney(double dblMoney) {
			if (dblMoney > 0) {
				dblAccountBalance = dblAccountBalance - dblMoney;
			} else {
				throw new BankOperationException(strAccountNum, "Account.takeMoney", "dblMoney <= 0");
			}
		}
		
		public void putMoney(double dblMoney) {
			if (dblMoney > 0) {
				dblAccountBalance = dblAccountBalance + dblMoney;
			} else {
				throw new BankOperationException(strAccountNum, "Account.putMoney", "dblMoney <= 0");
			}
		}
	}
	
	public class BankServer {
		internal protected Hashtable objAccounts;
				
		public BankServer(){
			objAccounts = new Hashtable();
			Console.WriteLine("BankServer 2.0.0.0");
		}
		
		public void createAccount(string strAccountNum){
			if (strAccountNum == null){
				throw new BankOperationException(strAccountNum, "BankServer.createAccount()", "strAccountNum is null!");
			} else if (objAccounts.Contains(strAccountNum)){
				throw new BankOperationException(strAccountNum, "BankServer.createAccount()", "strAccountNum exists!");
			} else {
				Account ac = new Account(strAccountNum);
				objAccounts.Add(strAccountNum, ac);
			}
		}
		
		public void deleteAccount(string strAccountNum){
			Account ac = getAccount(strAccountNum);
			if (ac.AccountBalance!=0){
				throw new BankOperationException(strAccountNum, "BankServer.deleteAccount()", "AccountBalance not 0!");
			} else {
				objAccounts.Remove(strAccountNum);
			}
		}
		
		public void payoutAccount(string strAccountNum, double dblMoney){
			Account ac = getAccount(strAccountNum);
			ac.takeMoney(dblMoney);
		}
		
		public void payinAccount(string strAccountNum, double dblMoney){
			Account ac = getAccount(strAccountNum);
			ac.putMoney(dblMoney);
		}
		
		public double checkAccountBalance(string strAccountNum){
			Account ac = getAccount(strAccountNum);
			return ac.AccountBalance;
		}
		
		private Account getAccount(string strAccountNum){
			if (strAccountNum == null){
				throw new BankOperationException(strAccountNum, "BankServer.getAccount()", "strAccountNum is null!");
			} else if (!objAccounts.Contains(strAccountNum)){
				throw new BankOperationException(strAccountNum, "BankServer.getAccount()", "strAccountNum does not exist!");
			} else {
				return (Account)objAccounts[strAccountNum];
			}
		}
	}
	
}