using System;
using System.ComponentModel;
using System.Threading;

namespace MyTimer {
	
	public delegate void TimeElapsedEvent();
	
	public class MyTimer : Component {
		protected int MilliTimeInterval = 0;
		protected Thread WorkerThread;
		public event TimeElapsedEvent TimeElapsed;
		
		public MyTimer(){
			ThreadStart threadStart = new ThreadStart(this.SleepingTimeInterval);
			WorkerThread = new Thread(threadStart);
			WorkerThread.Start();
		}
		
		[Description("Die Zeitdauer zwischen zwei Ereignissen")]
		public int TimeInterval {
			get {
				return MilliTimeInterval;
			}
			set {
				if (value >= 0){
					lock(this){
						MilliTimeInterval = value;
						Monitor.Pulse(this);
					}
				}
			}
		}
		
		protected override void Dispose(bool disposing){
			base.Dispose(disposing);
			try {
				if (WorkerThread != null) {
					WorkerThread.Interrupt();
					WorkerThread.Join();
				}
			} finally {}
		}
		
		private void SleepingTimeInterval(){
			bool pulsed;
			lock (this) {
				try {
					while(true){
						if (MilliTimeInterval == 0){
							pulsed = Monitor.Wait(this, Timeout.Infinite);
						} else {
							pulsed = Monitor.Wait(this, MilliTimeInterval);
						}
						if (pulsed == false){
							TimeElapsed();
						}
					}
				} catch (ThreadInterruptedException e){
				}
			}
		}
	}
}